"""
FCC plugin:


"""
import click

api_url = "https://fcc.io"


@click.group()
@click.pass_context
def plugins(ctx):
    """FCC plugin"""
    pass


@plugins.command()
@click.option("--fcc-id", help="FCC ID")
@click.pass_context
def search(ctx, fcc_id):
    """Search FCC by FCC ID"""
    if not fcc_id:
        click.echo(ctx.get_help())
        exit(0)
    click.launch(f"{api_url}/{fcc_id}")



import inspect

import pytest
from click.testing import CliRunner

from griffon.cli import cli
from griffon.services import Query, core_queries


@pytest.mark.unit
def test_queries():
    runner = CliRunner()
    result = runner.invoke(cli, ["service"])
    assert result.exit_code == 0


@pytest.mark.integration
def test_query_defs():
    for name, obj in inspect.getmembers(core_queries):
        if inspect.isclass(obj) and inspect.ismodule(obj):
            assert isinstance(obj, Query)
    assert isinstance(core_queries.product_stream_summary, Query)
    q: Query = core_queries.product_stream_summary()
    assert q



"""
    read only queries

"""
import concurrent
import logging
from typing import Any, Dict, List

from griffon import CORGI_API_URL, OSIDB_API_URL, CorgiService, OSIDBService

logger = logging.getLogger("griffon")


class product_stream_summary:
    """retrieve product_stream summary"""

    name = "product_stream_summary"
    description = "retrieve product_stream summary"
    allowed_params = ["strict_name_search", "product_stream_name", "ofuri"]

    def __init__(self, params: dict) -> None:
        self.corgi_session = CorgiService.create_session()
        self.params = params
        self.product_stream_name = self.params.get("product_stream_name")
        self.ofuri = self.params.get("ofuri")
        self.strict_name_search = self.params.get("strict_name_search", None)

    def execute(self) -> List[Dict[str, Any]]:
        cond = {}

        if self.ofuri:
            cond["ofuri"] = self.ofuri
        elif not self.strict_name_search:
            cond["re_name"] = self.product_stream_name
        else:
            cond["name"] = self.product_stream_name

        # TODO - corgi bindings need to support ofuri in product_streams
        product_streams = self.corgi_session.product_streams.retrieve_list(
            **cond,
            limit=400,
            include_fields="link,ofuri,name,products,product_versions,brew_tags,manifest",
        )
        results = []
        if product_streams.results:
            for ps in product_streams.results:
                result = {
                    "link": ps.link,
                    "ofuri": ps.ofuri,
                    "name": ps.name,
                    "product": ps.products[0]["name"],
                    "product_version": ps.product_versions[0]["name"],
                    "brew_tags": [brew_tag for brew_tag in ps.brew_tags.to_dict().keys()],
                    "build_count": ps.build_count,
                    "manifest_link": ps.manifest,
                    "latest_components_link": ps.components,
                    "all_components_link": f"{CORGI_API_URL}/api/v1/components?product_streams={ps.ofuri}&include_fields=link,name,purl",  # noqa
                }
                results.append(result)
        else:
            result = {
                "link": product_streams["link"],
                "ofuri": product_streams["ofuri"],
                "name": product_streams["name"],
                "product": product_streams["products"][0]["name"],
                "product_version": product_streams["product_versions"][0]["name"],
                "brew_tags": "",
                "manifest_link": product_streams["manifest"],
                "all_components_link": f"{CORGI_API_URL}/api/v1/components?product_streams={product_streams['ofuri']}&include_fields=link,name,purl",  # noqa
            }
            results.append(result)
        return results


class products_versions_affected_by_specific_cve_query:
    """Given a specific CVE ID, what products are affected?"""

    name = "product-versions-affected-by-specific-cve"
    description = "Given a specific CVE ID, what product versions are affected?"
    allowed_params = ["cve_id"]

    def __init__(self, params: dict) -> None:
        self.osidb_session = OSIDBService.create_session()
        self.params = params

    def execute(self) -> dict:
        cve_id = self.params["cve_id"]
        flaw = self.osidb_session.flaws.retrieve(cve_id)
        affects = list()
        pv_names = list()
        for affect in flaw.affects:
            pv_names.append(affect.ps_module)
            affects.append(
                {"component_name": affect.ps_component, "product_version_name": affect.ps_module}
            )
        pv_names = list(set(pv_names))
        product_versions = list()
        for pv_name in pv_names:
            product_versions.append(
                {
                    "link": f"{CORGI_API_URL}/api/v1/product_versions?name={pv_name}",
                    "name": pv_name,
                }
            )
        return {
            "link": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{cve_id}",
            "cve_id": cve_id,
            "title": flaw.title,
            "description": flaw.description,
            "product_versions": product_versions,
            "affects": affects,
        }


class products_containing_specific_component_query:
    """What products contain a specific component?"""

    name = "products_containing_specific_component_query"
    description = "What products contain a specific component?"
    allowed_params = [
        "component_name",
        "purl",
        "arch",
        "namespace",
        "component_type",
        "strict_name_search",
        "affect_mode",
        "search_deps",
    ]

    def __init__(self, params: dict) -> None:
        self.corgi_session = CorgiService.create_session()
        self.params = params
        self.purl = self.params.get("purl")

    def execute(self) -> dict:
        c = self.corgi_session.components.retrieve_list(
            purl=self.purl,
        )
        return c["product_streams"]


class products_containing_component_query:
    """What products contain a component?"""

    name = "products_containing_component_query"
    description = "What products contain a component?"
    allowed_params = [
        "component_name",
        "purl",
        "arch",
        "namespace",
        "component_type",
        "strict_name_search",
        "affect_mode",
        "search_deps",
    ]

    def __init__(self, params: dict) -> None:
        self.corgi_session = CorgiService.create_session()
        self.params = params
        self.component_name = self.params.get("component_name")
        self.component_type = self.params.get("component_type")
        self.strict_name_search = self.params.get("strict_name_search")
        self.search_deps = self.params.get("search_deps")
        self.ns = self.params.get("namespace")

    def execute(self) -> List[Dict[str, Any]]:
        cond = {"view": "latest"}
        if not self.strict_name_search:
            cond["re_name"] = self.component_name  # type: ignore
        else:
            cond["name"] = self.component_name  # type: ignore
        result = self.corgi_session.components.retrieve_list(**cond, limit=1000)
        return result.results


class components_containing_specific_component_query:
    """What components contain a specific component?"""

    name = "components_containing_specific_component_query"
    description = "What components contain a specific component?"
    allowed_params = [
        "component_re_name",
        "component_name",
        "purl",
        "component_type",
        "component_version",
        "component_arch",
        "namespace",
        "strict_name_search",
    ]

    def __init__(self, params: dict):
        self.corgi_session = CorgiService.create_session()
        self.params = params
        self.purl = self.params.get("purl")

    def execute(self) -> dict:
        if self.purl:
            c = self.corgi_session.components.retrieve_list(
                purl=self.purl,
            )
            component_type = self.params["component_type"]
            sources = c["sources"]
            if component_type:
                sources = [source for source in sources if component_type.lower() in source["purl"]]
        return {
            "link": c["link"],
            "type": c["type"],
            "name": c["name"],
            "version": c["version"],
            "purl": c["purl"],
            "sources": sources,
        }


class components_containing_component_query:
    """What components contain a component?"""

    name = "components_containing_component_query"
    description = "What components contain a component?"
    allowed_params = [
        "component_name",
        "purl",
        "component_type",
        "component_version",
        "component_arch",
        "namespace",
        "strict_name_search",
    ]

    def __init__(self, params: dict) -> None:
        self.corgi_session = CorgiService.create_session()
        self.params = params
        self.component_name = self.params.get("component_name")
        self.component_type = self.params.get("component_type")
        self.component_version = self.params.get("component_version")
        self.component_arch = self.params.get("component_arch")
        self.namespace = self.params.get("namespace")
        self.strict_name_search = self.params.get("strict_name_search")

    def execute(self) -> List[Dict[str, Any]]:
        cond = {}
        if not self.strict_name_search:
            cond["re_name"] = self.component_name
        else:
            cond["name"] = self.component_name
        if self.component_version:
            cond["version"] = self.component_version
        if self.component_arch:
            cond["arch"] = self.component_arch
        if self.namespace:
            cond["namespace"] = self.namespace

        components: List[Any] = []
        logger.debug("starting parallel http requests")
        component_cnt = self.corgi_session.components.retrieve_list(**cond).count
        if component_cnt < 3000000:
            with concurrent.futures.ThreadPoolExecutor() as executor:
                futures = []
                components = list()
                for batch in range(0, component_cnt, 120):
                    futures.append(
                        executor.submit(
                            self.corgi_session.components.retrieve_list,
                            **cond,
                            offset=batch,
                            include_fields="link,name,type,version,purl,sources",
                            limit=120,  # noqa
                        )
                    )
                for future in concurrent.futures.as_completed(futures):
                    try:
                        components.extend(future.result().results)
                    except Exception as exc:
                        logger.warning("%r generated an exception: %s" % (future, exc))

        results = []
        for c in components:
            sources = [{"link": source["link"], "purl": source["purl"]} for source in c.sources]
            if self.component_type:
                sources = [
                    source for source in sources if self.component_type.lower() in source["purl"]
                ]
            results.append(
                {
                    "link": c.link,
                    "type": c.type,
                    "name": c.name,
                    "version": c.version,
                    "purl": c.purl,
                    "sources": sources,
                }
            )
        return results


class components_affected_by_specific_cve_query:
    """Given a specific CVE ID, what components are affected?"""

    name = "components_affected_by_specific_cve_query"
    description = "Given a CVE ID, what components are affected?"
    allowed_params = [
        "cve_id",
        "affectedness",
        "affect_resolution",
        "affect_impact",
        "component_type",
        "namespace",
    ]

    def __init__(self, params: dict) -> None:
        self.corgi_session = CorgiService.create_session()
        self.osidb_session = OSIDBService.create_session()
        self.params = params
        self.cve_id = self.params.get("cve_id")
        self.affectedness = self.params.get("affectedness")
        self.affect_resolution = self.params.get("affect_resolution")
        self.affect_impact = self.params.get("affect_impact")
        self.component_type = self.params.get("component_type")
        self.namespace = self.params.get("namespace")

    def execute(self) -> dict:
        cond = {}
        if self.affectedness:
            cond["affectedness"] = self.affectedness
        if self.affect_resolution:
            cond["resolution"] = self.affect_resolution
        if self.affect_impact:
            cond["impact"] = self.affect_impact

        component_cond = {}
        if self.namespace:
            component_cond["namespace"] = self.namespace
        if self.component_type:
            component_cond["type"] = self.component_type

        flaw = self.osidb_session.flaws.retrieve(self.cve_id)
        affects = self.osidb_session.affects.retrieve_list(
            flaw=flaw.uuid, **cond, limit=1000
        ).results
        results = list()
        for affect in affects:
            try:
                product_version = self.corgi_session.product_versions.retrieve_list(
                    name=affect.ps_module
                ).results[0]
                components = []
                with concurrent.futures.ThreadPoolExecutor() as executor:
                    futures = []
                    for ps in product_version.product_streams:
                        futures.append(
                            executor.submit(
                                self.corgi_session.components.retrieve_list,
                                **component_cond,
                                ofuri=ps["ofuri"],
                                name=affect.ps_component,
                                include_fields="link,purl,name,type",
                                limit=50000,
                            )
                        )
                    for future in concurrent.futures.as_completed(futures):
                        try:
                            for c in future.result().results:
                                components.append(c.to_dict())
                        except Exception as exc:
                            logger.error("%r generated an exception: %s" % (future, exc))
                            exit(0)

                results.append(
                    {
                        "link": f"{OSIDB_API_URL}/osidb/api/v1/affects/{affect.uuid}",
                        "product_version_name": affect.ps_module,
                        "component_name": affect.ps_component,
                        "affectedness": affect.affectedness,
                        "affect_impact": affect.impact,
                        "affect_resolution": affect.resolution,
                        "components": components,
                    }
                )
            except IndexError:
                logger.warning(
                    f"{affect.ps_module} product stream not found in component-registry (may not exist or a community product)."  # noqa
                )

        return {
            "link": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{flaw.cve_id}",
            "cve_id": flaw.cve_id,
            "title": flaw.title,
            "description": flaw.description,
            "affects": results,
        }


class cves_for_specific_component_query:
    """CVEs affecting a specific component?"""

    # include count
    name = "cves_for_specific component"
    description = "Which CVEs affect a specific component ?"
    allowed_params = [
        "component_name",
        "purl",
        "flaw_state",
        "flaw_impact",
        "flaw_resolution",
        "affectedness",
        "affect_resolution",
        "affect_impact",
        "strict_name_search",
    ]

    def __init__(self, params: dict) -> None:
        self.corgi_session = CorgiService.create_session()
        self.osidb_session = OSIDBService.create_session()
        self.params = params
        self.component_name = self.params.get("component_name")
        self.purl = self.params.get("purl")
        self.flaw_state = self.params.get("flaw_state")
        self.flaw_impact = self.params.get("flaw_impact")
        self.flaw_resolution = self.params.get("flaw_resolution")
        self.affectedness = self.params.get("affectedness")
        self.affect_resolution = self.params.get("affect_resolution")
        self.affect_impact = self.params.get("affect_impact")

    def execute(self) -> List[Dict[str, Any]]:
        components = []
        if self.component_name:
            affects: list = []
            # component = self.corgi_session.components.retrieve_list(name=self.component_name)
            cond = {}
            cond["ps_component"] = self.component_name
            if self.affectedness:
                cond["affectedness"] = self.affectedness
            if self.affect_resolution:
                cond["resolution"] = self.affect_resolution
            if self.affect_impact:
                cond["impact"] = self.affect_impact

            for affect in self.osidb_session.affects.retrieve_list(
                **cond,
                limit=50,
            ).results:
                flaw = self.osidb_session.flaws.retrieve(affect.flaw)
                if flaw:
                    affects.append(
                        {
                            "link_affect": f"{OSIDB_API_URL}/osidb/api/v1/affects/{affect.uuid}",  # noqa
                            "link_cve": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{flaw.cve_id}",
                            "flaw_cve_id": flaw.cve_id,
                            "title": flaw.title,
                            "flaw_state": flaw.state,
                            "flaw_resolution": flaw.resolution,
                            "affect_name": affect.ps_component,
                            "affect_product_version": affect.ps_module,
                            "affect_affectedness": affect.affectedness,
                            "affect_impact": affect.impact,
                            "affect_resolution": affect.resolution,
                        }
                    )
            components.append(
                {
                    "link": f"{CORGI_API_URL}/api/v1/components?purl=",
                    "name": self.component_name,
                    "affects": affects,
                }
            )

        if self.purl:
            pass

        return components


class cves_for_specific_product_query:
    name = "cves-for-product"
    description = "What cves affect a specific product ?"
    allowed_params = [
        "product_version_name",
        "ofuri",
        "flaw_state",
        "flaw_impact",
        "flaw_resolution",
        "affectedness",
        "affect_resolution",
        "affect_impact",
        "strict_name_search",
    ]

    def __init__(self, params: dict) -> None:
        self.corgi_session = CorgiService.create_session()
        self.osidb_session = OSIDBService.create_session()
        self.params = params
        self.product_version_name = self.params.get("product_version_name")
        self.ofuri = self.params.get("ofuri")
        self.flaw_state = self.params.get("flaw_state")
        self.flaw_impact = self.params.get("flaw_impact")
        self.flaw_resolution = self.params.get("flaw_resolution")
        self.affectedness = self.params.get("affectedness")
        self.affect_resolution = self.params.get("affect_resolution")
        self.affect_impact = self.params.get("affect_impact")

    def execute(self) -> List[Dict[str, Any]]:
        components = []
        if self.product_version_name:
            affects: list = []
            # component = self.corgi_session.components.retrieve_list(name=self.component_name)
            cond = {}
            cond["ps_module"] = self.product_version_name
            if self.affectedness:
                cond["affectedness"] = self.affectedness
            if self.affect_resolution:
                cond["resolution"] = self.affect_resolution
            if self.affect_impact:
                cond["impact"] = self.affect_impact

            for affect in self.osidb_session.affects.retrieve_list(
                **cond,
                limit=10,
            ).results:
                flaw = self.osidb_session.flaws.retrieve(affect.flaw)
                if flaw:
                    affects.append(
                        {
                            "link_affect": f"{OSIDB_API_URL}/osidb/api/v1/affects/{affect.uuid}",  # noqa
                            "link_cve": f"{OSIDB_API_URL}/osidb/api/v1/flaws/{flaw.cve_id}",
                            "flaw_cve_id": flaw.cve_id,
                            "title": flaw.title,
                            "flaw_state": flaw.state,
                            "flaw_resolution": flaw.resolution,
                            "affect_name": affect.ps_component,
                            "affect_product_version": affect.ps_module,
                            "affect_affectedness": affect.affectedness,
                            "affect_impact": affect.impact,
                            "affect_resolution": affect.resolution,
                        }
                    )
            components.append(
                {
                    "link": f"{CORGI_API_URL}/api/v1/product_version?ofuri=",
                    "name": self.product_version_name,
                    "affects": affects,
                }
            )

        if self.ofuri:
            pass

        return components



"""

"""
import logging

import click

from griffon import CORGI_API_URL, OSIDB_API_URL, CorgiService, OSIDBService
from griffon.output import console, cprint

logger = logging.getLogger("griffon")


@click.group(name="manage")
@click.pass_context
def manage_grp(ctx):
    """Manage operations."""
    pass


# osidb manage commands
@manage_grp.group()
@click.pass_context
def osidb(ctx):
    """Manage osidb"""
    pass


@osidb.command(name="status")
@click.pass_context
def osidb_status(ctx):
    session = OSIDBService.create_session()
    data = session.status()
    return cprint(data, ctx=ctx)


@osidb.command(name="api_doc")
def osidb_api_docs():
    click.launch(f"{OSIDB_API_URL}/osidb/api/v1/schema/swagger-ui/")


# component-registry manage commands
@manage_grp.group()
@click.pass_context
def corgi(ctx):
    """Manage component registry"""
    pass


@corgi.command(name="status")
@click.pass_context
def corgi_status(ctx):
    session = CorgiService.create_session()
    data = session.status()
    return cprint(data.additional_properties, ctx=ctx)


@corgi.command(name="health")
@click.pass_context
def corgi_health(ctx):
    try:
        session = CorgiService.create_session()
        status = session.status()["status"]
        if status == "ok":
            console.log(f"{CORGI_API_URL} is operational")
        else:
            console.log(f"{CORGI_API_URL} is NOT operational")
            exit(1)
    except:  # noqa
        console.log(f"{CORGI_API_URL} is NOT operational")
        raise click.ClickException("Component registry health check failed.")


@corgi.command(name="data")
def corgi_data():
    click.launch(f"{CORGI_API_URL}/data")


@corgi.command(name="api_doc")
def corgi_api_docs():
    click.launch(f"{CORGI_API_URL}/api/v1/schema/docs")



import configparser
import logging
import os
from configparser import ConfigParser
from functools import partial, wraps

import component_registry_bindings
import osidb_bindings
from rich.logging import RichHandler

from griffon.output import console

__version__ = "0.0.7"

CORGI_API_URL = os.environ["CORGI_API_URL"]
OSIDB_API_URL = os.environ["OSIDB_API_URL"]

GRIFFON_CONFIG_DIR = os.getenv("GRIFFON_API_URL", "~/.griffon")
GRIFFON_RC_FILE = "~/.griffonrc"
GRIFFON_DEFAULT_LOG_FILE = "~/.griffon/griffon.log"

logging.basicConfig(level="INFO")
logger = logging.getLogger("griffon")

# file_handler = logging.FileHandler(os.path.expanduser(GRIFFON_DEFAULT_LOG_FILE))
formatter = logging.Formatter("%(asctime)s %(name)s %(levelname)s %(message)s")
# file_handler.setFormatter(formatter)
# logger.addHandler(file_handler)
logger.handlers = [RichHandler()]

logger = logging.getLogger("griffon")


def get_config():
    """read ~/.griffonrc ini file, if it does not exist then return some default config"""
    if not os.path.exists(os.path.expanduser(GRIFFON_RC_FILE)):
        config = configparser.ConfigParser(allow_no_value=True)
        config.optionxform = str
        config.add_section("default")
        config["default"]["log_file"] = GRIFFON_DEFAULT_LOG_FILE
        config["default"]["format"] = "text"
        config.add_section("exclude")
        return config
    config = ConfigParser()
    config.read(os.path.expanduser(GRIFFON_RC_FILE))
    return config


class CorgiService:
    name = "component-registry"
    description = "Red Hat component registry"
    has_binding = True

    @staticmethod
    def create_session():
        """init corgi session"""
        try:
            return component_registry_bindings.new_session(
                component_registry_server_uri=CORGI_API_URL
            )
        except:  # noqa
            console.log(f"{CORGI_API_URL} is not accessible.")
            exit(1)

    @staticmethod
    def get_component_types():
        """get component type enum"""
        return (
            component_registry_bindings.bindings.python_client.models.component_type_enum.ComponentTypeEnum  # noqa
        )

    @staticmethod
    def get_component_namespaces():
        """get component namespaces enum"""
        return (
            component_registry_bindings.bindings.python_client.models.namespace_enum.NamespaceEnum
        )

    @staticmethod
    def get_component_arches():
        """get component arch enum"""
        return [
            "src",
            "noarch",
            "i386",
            "ia64",
            "s390",
            "x86_64",
            "s390x",
            "ppc",
            "ppc64",
            "aarch64",
            "ppc64le",
        ]


class OSIDBService:
    name = "osidb"
    description = "Open Source Incident database"
    has_binding = True

    @staticmethod
    def create_session():
        """init osidb session"""
        try:
            return osidb_bindings.new_session(osidb_server_uri=OSIDB_API_URL)
        except:  # noqa
            console.log(f"{OSIDB_API_URL} is not accessible (or krb ticket has expired).")
            exit(1)

    @staticmethod
    def get_flaw_states():
        """get flaw states enum"""
        return osidb_bindings.bindings.python_client.models.FlawClassificationState

    @staticmethod
    def get_flaw_resolutions():
        """get flaw resolution enum"""
        return osidb_bindings.bindings.python_client.models.FlawResolutionEnum

    @staticmethod
    def get_flaw_impacts():
        """get flaw impacts enum"""
        return osidb_bindings.bindings.python_client.models.ImpactEnum

    @staticmethod
    def get_affect_affectedness():
        """get affect affectedness enum"""
        return osidb_bindings.bindings.python_client.models.AffectednessEnum

    @staticmethod
    def get_affect_resolution():
        """get affect affectedness enum"""
        return osidb_bindings.bindings.python_client.models.AffectResolutionEnum

    @staticmethod
    def get_affect_impact():
        """get affect impact enum"""
        return osidb_bindings.bindings.python_client.models.ImpactEnum


def progress_bar(
    func=None,
):
    """progress bar decorator"""
    if not func:
        return partial(progress_bar)

    @wraps(func)
    def wrapper(*args, **kwargs):
        obj: dict = args[0].obj
        if obj.get("NO_PROGRESS_BAR"):
            func(*args, **kwargs)
        else:
            with console.status("griffoning", spinner="line"):
                func(*args, **kwargs)

    return wrapper


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    print(f"Version {__version__}")
    ctx.exit()



import pytest
from click.testing import CliRunner

from griffon.cli import cli

pytestmark = pytest.mark.unit


def test_cli_flaws():
    runner = CliRunner()
    result = runner.invoke(cli, ["entities", "flaws"])
    assert result.exit_code == 0


def test_cli_components():
    runner = CliRunner()
    result = runner.invoke(cli, ["entities", "components"])
    assert result.exit_code == 0



"""

"""
import configparser
import logging
import os

import click

from griffon import GRIFFON_CONFIG_DIR, GRIFFON_DEFAULT_LOG_FILE, GRIFFON_RC_FILE

logger = logging.getLogger("griffon")


@click.group(name="configure")
@click.pass_context
def configure_grp(ctx):
    """Configure griffon."""
    pass


@configure_grp.command(name="setup", help="Create ~/.griffon and .griffonrc config file")
def setup():
    """stub"""
    if not os.path.exists(os.path.expanduser(GRIFFON_CONFIG_DIR)):
        os.makedirs(os.path.expanduser(GRIFFON_CONFIG_DIR))
        logger.warning(f"{GRIFFON_CONFIG_DIR} created")
    else:
        logger.warning(f"{GRIFFON_CONFIG_DIR} already exists")

    config = configparser.ConfigParser(allow_no_value=True)
    config.optionxform = str
    config.add_section("default")
    config["default"]["log_file"] = GRIFFON_DEFAULT_LOG_FILE
    config["default"]["format"] = "text"
    config.add_section("exclude")
    with open(os.path.expanduser(GRIFFON_RC_FILE), "w") as configfile:
        config.write(configfile)



import click
import pytest

from griffon.commands.queries import product_versions_affected_by_cve_query
from griffon.output import OUTPUT_FORMAT, cprint

pytestmark = pytest.mark.unit


def test_output_formats():

    assert OUTPUT_FORMAT("json") == OUTPUT_FORMAT.JSON
    assert OUTPUT_FORMAT.TEXT.value == "text"

    single_result_data = {
        "ofuri": "o:redhat:ansible_automation_platform:2.2",
        "name": "ansible_automation_platform-2.2",
        "product": "ansible-automation-platform",
        "product_version": "ansible_automation_platform-2",
        "brew_tags": [
            "ansible-automation-platform-2.2-rhel-8",
            "ansible-automation-platform-2.2-rhel-9",
            "ansible-automation-platform-2.2-rhel-8-container-released",
            "ansible-automation-platform-2.2-rhel-9-container-released",
        ],
        "build_count": 1033,
        "latest_component_count": 246,
    }
    with pytest.raises(SystemExit) as capture_err:
        ctx = click.Context(
            product_versions_affected_by_cve_query,
            obj={
                "NO_COLOR": False,
                "NO_PROGRESS_BAR": False,
                "SHOW_INACTIVE": False,
                "SHOW_UPSTREAM": False,
            },
        )
        assert cprint(single_result_data, ctx=ctx)
    assert capture_err
    assert capture_err.type == SystemExit
    assert capture_err.value.code == 0



import enum
import json
import logging

import click
from packageurl import PackageURL
from rich.console import Console
from rich.text import Text

console = Console(color_system="auto")

logger = logging.getLogger("griffon")


class OUTPUT_FORMAT(enum.Enum):
    JSON = "json"
    TEXT = "text"
    TABLE = "table"


class DEST(enum.Enum):
    CONSOLE = "console"
    FILE = "file"


def raw_json_transform(data, show_count: bool) -> dict:
    """normalise all data to dict"""
    if type(data) is list:
        results = []
        for d in data:
            if type(d) is dict:
                results.append(d)
            else:
                results.append(d.to_dict())
        output = {
            "results": results,
        }
        if show_count:
            output["count"] = len(results)  # type: ignore
    else:
        if type(data) is dict:
            output = data
        else:
            output = data.to_dict()
    return output


def entity_type(data):
    entity_type = "unknown"
    if "cve_id" in data:
        entity_type = "flaw"
    if "purl" in data:
        entity_type = "component"
    if "affectedness" in data:
        entity_type = "affect"
    if "ofuri" in data:
        entity_type = "product"
    return entity_type


def component_type_style(type):
    from griffon import CorgiService

    types = [type.value for type in CorgiService.get_component_types()]
    colors = (
        "cornflower_blue",
        "red",
        "dark_slate_gray1",
        "magenta",
        "navy_blue",
        "green1",
        "blue",
        "dark_orange",
        "deep_pink1",
        "red1",
        "green1",
    )
    color = colors[types.index(type)]
    return f"[{color}]{type}[/{color}]"


def text_output_product_summary(ctx, output, format):

    ordered_results = sorted(output["results"], key=lambda d: d["name"])

    if ctx.obj["VERBOSE"] == 0:
        for item in ordered_results:
            console.print(
                Text(item["product"], style="bold magenta u"),
                Text(item["product_version"], style="magenta"),
                Text(item["name"], style="white"),
                f"{item['brew_tags']}",
                no_wrap=False,
            )
    if ctx.obj["VERBOSE"] == 1:
        for item in ordered_results:
            console.print(
                Text(item["product"], style="bold magenta u"),
                Text(item["product_version"], style="magenta"),
                Text(item["name"], style="white"),
                item["brew_tags"],
                item["ofuri"],
                no_wrap=False,
            )
    if ctx.obj["VERBOSE"] > 1:
        for item in ordered_results:
            console.print(
                Text(item["product"], style="bold magenta u"),
                Text(item["product_version"], style="magenta"),
                Text(item["name"], style="white"),
                item["brew_tags"],
                item["ofuri"],
                item["manifest_link"],
                no_wrap=False,
            )
    ctx.exit()


def text_output_products_contain_component(ctx, output, format):
    component_name = ctx.params["component_name"]

    # handle single component
    if ctx.params["purl"]:
        ordered_results = sorted(output["results"], key=lambda d: d["ofuri"])
        for item in ordered_results:
            console.print(
                Text(item["ofuri"], style="bold magenta u"),
                no_wrap=False,
            )
        ctx.exit()

    # handle multiple components
    if "results" in output and output["count"] > 0:
        ordered_results = sorted(output["results"], key=lambda d: d["product_stream"])

        if ctx.obj["VERBOSE"] == 0:  # product_version X source component
            product_versions = sorted(
                list(set([item["product_version"] for item in ordered_results]))
            )
            for pv in product_versions:
                names = []
                for item in ordered_results:
                    if pv == item["product_version"]:
                        names.append(item["name"])
                names = list(set(names))
                for name in names:
                    dep_name = name.replace(component_name, f"[b]{component_name}[/b]")
                    dep = f"[white]({dep_name})[/white]"
                    console.print(
                        Text(pv, style="magenta b u"),
                        dep,
                        no_wrap=False,
                    )
        if ctx.obj["VERBOSE"] == 1:  # product_stream X source component
            for item in ordered_results:
                dep_name = item["name"].replace(component_name, f"[b]{component_name}[/b]")
                dep = f"[white]({dep_name})[/white]"
                root_component = "[i]Root component[/i]"
                if item.get("root_component"):
                    root_component = item["root_component"]
                console.print(
                    Text(item["product_stream"], style="magenta b u"),
                    root_component,
                    dep,
                    no_wrap=False,
                )
        if ctx.obj["VERBOSE"] == 2:  # product_stream X nvr
            for item in ordered_results:
                dep_name = item["nvr"].replace(component_name, f"[b]{component_name}[/b]")
                dep = f"[white]({dep_name})[/white]"
                root_component = "[i]Root component[/i]"
                if item.get("root_component"):
                    root_component = item["root_component"]
                console.print(
                    Text(item["product_stream"], style="magenta b u"),
                    root_component,
                    dep,
                    no_wrap=False,
                )
        if ctx.obj["VERBOSE"] == 3:  # related url
            for item in ordered_results:
                dep_name = item["nvr"].replace(component_name, f"[b]{component_name}[/b]")
                dep = f"[white]({dep_name})[/white]"
                root_component = "[i]Root component[/i]"
                if item.get("root_component"):
                    root_component = item["root_component"]
                console.print(
                    Text(item["product_stream"], style="magenta b u"),
                    root_component,
                    dep,
                    item["related_url"],
                    no_wrap=False,
                )
        if ctx.obj["VERBOSE"] == 4:  # source url
            for item in ordered_results:
                dep_name = item["nvr"].replace(component_name, f"[b]{component_name}[/b]")
                dep = f"[white]({dep_name})[/white]"
                root_component = "[i]Root component[/i]"
                if item.get("root_component"):
                    root_component = item["root_component"]
                console.print(
                    Text(item["product_stream"], style="magenta b u"),
                    root_component,
                    dep,
                    item["related_url"],
                    item["build_source_url"],
                    no_wrap=False,
                )
        if ctx.obj["VERBOSE"] > 4:  # source url, upstream
            for item in ordered_results:
                dep_name = item["nvr"].replace(component_name, f"[b]{component_name}[/b]")
                dep = f"[white]({dep_name})[/white]"
                root_component = "[i]Root component[/i]"
                if item.get("root_component"):
                    root_component = item["root_component"]
                console.print(
                    Text(item["product_stream"], style="magenta b u"),
                    root_component,
                    dep,
                    item["related_url"],
                    item["build_source_url"],
                    item["upstream_purl"],
                    no_wrap=False,
                )
        ctx.exit()


def text_output_components_contain_component(ctx, output, format):
    if "results" in output:

        for item in output["results"]:
            component_name = item["name"]

            if ctx.obj["VERBOSE"] == 0:
                ordered_sources = sorted(item["sources"], key=lambda d: d["purl"])
                for source in ordered_sources:
                    if "arch=noarch" in source["purl"] or "arch=src" in source["purl"]:
                        source_purl = PackageURL.from_string(source["purl"])
                        root_component = source_purl.name
                        if source_purl.type == "oci" and "-source" not in source_purl.name:
                            root_component = f"[u magenta]{source_purl.name}-container[/u magenta]"
                        console.print(
                            root_component,
                            component_name,
                            no_wrap=False,
                        )
    ctx.exit()


def text_output_components_affected_by_cve(ctx, output, format):
    console.print("Flaw Title:", output["title"])
    console.print(
        "affects:",
    )
    ordered_affects = sorted(output["affects"], key=lambda d: d["product_version_name"])
    for affect in ordered_affects:
        if "components" in affect:
            for component in affect["components"]:
                affected_component1 = f"({component['purl']})"
                if not ctx.obj["SHOW_PURL"]:
                    purl = PackageURL.from_string(component["purl"])
                    ns = "UPSTREAM"
                    if purl.namespace:
                        ns = purl.namespace.upper()
                    affected_component1 = f"([bold cyan]{ns}[/bold cyan] {purl.name}-{purl.version},{purl.type.upper()})"  # noqa
                    if ctx.obj["VERBOSE"] == 0:
                        console.print(
                            ns,
                            affected_component1,
                            no_wrap=True,
                        )
                    if ctx.obj["VERBOSE"] == 1:
                        console.print(
                            ns,
                            affected_component1,
                            no_wrap=True,
                        )
    ctx.exit()


def text_output_products_affected_by_cve(ctx, output, format):
    console.print("[white]link:[/white]", output["link"])
    console.print("[white]cve_id:[/white]", output["cve_id"])
    console.print("[white]title:[/white]", output["title"])
    console.print(
        "[white]product_versions:[/white]",
    )
    if ctx.obj["VERBOSE"] == 0:
        ordered_product_versions = sorted(output["product_versions"], key=lambda d: d["name"])
        for product_version in ordered_product_versions:
            console.print(Text(product_version["name"], style="bold magenta u"), no_wrap=True)
    if ctx.obj["VERBOSE"] > 0:
        ordered_affects = sorted(output["affects"], key=lambda d: d["product_version_name"])
        for affect in ordered_affects:
            console.print(
                Text(affect["product_version_name"], style="bold magenta u"),
                affect["component_name"],
                no_wrap=True,
            )
    ctx.exit()


def text_output_get_manifest(ctx, output, format):
    if not ctx.obj["SHOW_PURL"]:
        for component in output["packages"]:
            if "pkg:" in component["externalRefs"][0]["referenceLocator"]:
                purl = PackageURL.from_string(component["externalRefs"][0]["referenceLocator"])
                ns = "[cyan]UPSTREAM[/cyan]"
                component = f"([bold turquoise2]{ns}[/bold turquoise2] [white]{purl.name}-{purl.version}[/white],{component_type_style(purl.type.upper())})"  # noqa
                if purl.namespace == "redhat":
                    ns = f"[red]{purl.namespace.upper()}[/red]"
                    component = f"([white]{purl.name}-{purl.version}[/white],{component_type_style(purl.type.upper())})"  # noqa
                else:
                    if purl.namespace:
                        ns = f"[white]{purl.namespace.upper()}[/white]"
                    component = f"([white]{purl.name}-{purl.version}[/white],{component_type_style(purl.type.upper())})"  # noqa
                console.print(ns, component, no_wrap=False)  # noqa
    else:
        for component in output["packages"]:
            purl = component["externalRefs"][0]["referenceLocator"]
            console.print(purl, no_wrap=False)  # noqa

    ctx.exit()


def text_output_component_flaws(ctx, output, format):
    ordered_components = sorted(output["results"], key=lambda d: d["name"])
    for item in ordered_components:
        component_name = item["name"]
        # sorting should work when there is no title or cve-id key
        ordered_affects = sorted(item["affects"], key=lambda d: (d["title"] is None, d["title"]))
        for affect in ordered_affects:
            flaw_cve_id = "Vulnerability"
            if affect["flaw_cve_id"]:
                flaw_cve_id = affect["flaw_cve_id"]
            if ctx.obj["VERBOSE"] == 0:
                console.print(
                    Text(flaw_cve_id, style="magenta"),
                    Text(component_name, style="white"),
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
            if ctx.obj["VERBOSE"] == 1:
                console.print(
                    Text(flaw_cve_id, style="magenta"),
                    Text(component_name, style="white"),
                    f"(state: {affect['flaw_state']} resolution:{affect['flaw_resolution']})",
                    Text(affect["affect_product_version"], style="cyan"),
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
            if ctx.obj["VERBOSE"] > 1:
                console.print(Text(affect["title"], style="white"))
                console.print(
                    Text(component_name, style="magenta"),
                    f"(state: {affect['flaw_state']} resolution:{affect['flaw_resolution']})",
                    Text(affect["affect_product_version"], style="cyan"),
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
    ctx.exit()


def text_output_product_flaws(ctx, output, format):
    for item in output["results"]:
        component_name = item["name"]
        for affect in item["affects"]:
            if ctx.obj["VERBOSE"] == 0:
                console.print(
                    Text(affect["flaw_cve_id"], style="magenta"),
                    Text(component_name, style="white"),
                    affect["affect_name"],
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
            if ctx.obj["VERBOSE"] == 1:
                console.print(
                    Text(affect["flaw_cve_id"], style="magenta"),
                    Text(component_name, style="white"),
                    f"(state: {affect['flaw_state']} resolution:{affect['flaw_resolution']})",
                    affect["affect_name"],
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
            if ctx.obj["VERBOSE"] > 1:
                console.print(affect["title"])
                console.print(
                    Text(component_name, style="white"),
                    f"(state: {affect['flaw_state']} resolution:{affect['flaw_resolution']})",
                    affect["affect_name"],
                    affect["affect_affectedness"],
                    affect["affect_impact"],
                    affect["affect_resolution"],
                    no_wrap=True,
                )
    ctx.exit()


def text_output_list(ctx, output, format):
    if "results" in output and output["count"] > 0:

        # handle component
        if "purl" in output["results"][0]:
            ordered_components = sorted(output["results"], key=lambda d: d["name"])
            for row in ordered_components:
                if "purl" in row:
                    purl = PackageURL.from_string(row["purl"])
                    if not purl.namespace:
                        component_ns = Text("UPSTREAM", style="bold magenta")
                    else:
                        component_ns = Text(purl.namespace.upper(), style="bold red")

                    if not ctx.obj["SHOW_PURL"]:
                        console.print(
                            component_ns,
                            purl.type.upper(),
                            Text(purl.name, style="bold white"),
                            purl.version,
                            row["related_url"],
                            purl.qualifiers.get("arch"),
                        )
                    else:
                        console.print(
                            row["purl"],
                        )
        # handle flaw
        if "cve_id" in output["results"][0]:
            for row in output["results"]:
                console.print(
                    row["title"],
                    row["state"],
                    row["impact"],
                    row["resolution"],
                    no_wrap=True,
                )

        # handle trackers
        if "external_system_id" in output["results"][0]:
            for row in output["results"]:
                console.print(
                    row["external_system_id"],
                    row["type"],
                    row["status"],
                    no_wrap=True,
                )

        # handle products
        if "ofuri" in output["results"][0]:
            for row in output["results"]:
                console.print(
                    Text(row["name"], style="magenta bold u"),
                    row["ofuri"],
                    no_wrap=True,
                )
    ctx.exit()


def text_output_generic(ctx, output, format):
    for k, v in output.items():
        key_name = Text(k)
        key_name.stylize("bold magenta")
        console.print(key_name, " : ", v, no_wrap=True)


def cprint(
    data,
    dest=DEST.CONSOLE,
    filename=None,
    ctx=None,
    show_count: bool = True,
):
    """handle format and output"""
    output = raw_json_transform(data, show_count)
    console = Console(color_system="auto")
    if ctx.obj and ctx.obj["NO_COLOR"]:
        console = Console(color_system=None)
    format = OUTPUT_FORMAT.JSON
    if ctx and "FORMAT" in ctx.obj:
        format = OUTPUT_FORMAT(ctx.obj["FORMAT"])

    if format is OUTPUT_FORMAT.TEXT:
        if ctx.info_name == "product-summary":
            text_output_product_summary(ctx, output, format)
        if ctx.info_name == "products-contain-component":
            text_output_products_contain_component(ctx, output, format)
        if ctx.info_name == "components-contain-component":
            text_output_components_contain_component(ctx, output, format)
        if ctx.info_name == "components-affected-by-cve":
            text_output_components_affected_by_cve(ctx, output, format)
        if ctx.info_name == "products-affected-by-cve":
            text_output_products_affected_by_cve(ctx, output, format)
        if ctx.info_name == "get-manifest":
            text_output_get_manifest(ctx, output, format)
        if ctx.info_name == "list":
            text_output_list(ctx, output, format)
        if ctx.info_name == "component-flaws":
            text_output_component_flaws(ctx, output, format)
        if ctx.info_name == "product-flaws":
            text_output_product_flaws(ctx, output, format)

        # last chance text formatted output
        text_output_generic(ctx, output, format)

    if format is OUTPUT_FORMAT.JSON:
        if dest is DEST.CONSOLE:
            console.print_json(json.dumps(output))

    # if we instructed to open browser, open that up now
    if ctx:
        if "link" in data and "open_browser" in ctx.obj:
            if ctx.obj["open_browser"]:
                click.launch(data["link"])

    exit(0)



"""
mangen plugin:


"""
import logging
import os
import subprocess
from configparser import ConfigParser

import click
import requests

from griffon import GRIFFON_CONFIG_DIR
from griffon.output import console

logger = logging.getLogger("griffon")

api_url = "https://vuln.go.dev"


# define autocomplete for service names
@click.group(help="(UNDER DEV) mangen plugin")
@click.pass_context
def plugins(ctx):
    """MANGEN plugin"""
    pass


@plugins.command(help="Run this first")
@click.pass_context
def setup(ctx):
    if not os.path.exists(os.path.expanduser(f"{GRIFFON_CONFIG_DIR}/repos")):
        os.makedirs(os.path.expanduser(f"{GRIFFON_CONFIG_DIR}/repos"))
    subprocess.run(
        ["git", "clone", "https://git.prodsec.redhat.com/prodsec/manifests.git"],
        cwd=os.path.expanduser(f"{GRIFFON_CONFIG_DIR}/repos"),
    )


@plugins.command()
@click.option("--name", "service_name")
@click.pass_context
def get(ctx, service_name):
    """ """
    if not service_name:
        click.echo(ctx.get_help())
        exit(0)

    if service_name:
        config = ConfigParser()
        config.read("/tmp/mangen.ini")

        config_dict = {}
        for section in config.sections():
            config_dict[section] = {}
            for option in config.options(section):
                config_dict[section][option] = config.get(section, option)

        res = requests.get(config_dict[service_name]["url"])
        console.print(res.text)



"""
Mozila advisories plugin:

    https://github.com/mozilla/foundation-security-advisories plugin

Run the following to setup the plugin (installs pyyaml and pulls git repo)
> griffon z_mfsa setup

"""
import logging
import os
import subprocess

import click

from griffon import GRIFFON_CONFIG_DIR

logger = logging.getLogger("griffon")

api_url = "https://api.osv.dev/v1/query"


@click.group(help="(UNDER DEV) MFSA plugin")
@click.pass_context
def plugins(ctx):
    """https://github.com/mozilla/foundation-security-advisories plugin"""
    pass


@plugins.command()
@click.pass_context
def update(ctx):
    subprocess.run(
        [
            "git",
            "log",
            "--graph",
            "--pretty=format:'%Cred%h%Creset -%C(yellow)%d%Creset %s %Cgreen(%cr) %C(bold blue)<%an>%Creset'",  # noqa
            "--abbrev-commit",
        ],
        cwd=os.path.expanduser("~/.griffon/repos/foundation-security-advisories"),
    )


@plugins.command(help="Run this first, installs pyyaml amd pulls repo")
@click.pass_context
def setup(ctx):
    subprocess.run(["pip", "install", "pyyaml"])
    if not os.path.exists(os.path.expanduser(f"{GRIFFON_CONFIG_DIR}/repos")):
        os.makedirs(os.path.expanduser(f"{GRIFFON_CONFIG_DIR}/repos"))
    subprocess.run(
        ["git", "clone", "https://github.com/mozilla/foundation-security-advisories.git"],
        cwd=os.path.expanduser(f"{GRIFFON_CONFIG_DIR}/repos"),
    )


@plugins.command()
@click.pass_context
def advisories(ctx):
    pass


@plugins.command()
@click.pass_context
def get(ctx):
    pass


@plugins.command()
@click.pass_context
def search(ctx):
    pass



"""
    read only queries

"""
import logging
from datetime import datetime

import requests

from griffon import OSIDB_API_URL, CorgiService, OSIDBService

logger = logging.getLogger("griffon")


class example_affects_report:
    """ """

    name = "example_affects_report"
    description = " "
    allowed_params = ["show_components", "show_products", "purl", "name", "ofuri", "product_name"]

    def __init__(self, params) -> None:
        self.corgi_session = CorgiService.create_session()
        self.osidb_session = OSIDBService.create_session()
        self.params = params

    def generate(self) -> dict:

        show_components = self.params["show_components"]
        show_products = self.params["show_products"]
        purl = self.params["purl"]
        component_name = self.params["name"]
        ofuri = self.params["ofuri"]
        product_name = self.params["product_name"]

        affects = self.osidb_session.affects.retrieve_list()

        affects_affected = self.osidb_session.affects.retrieve_list(affectedness="AFFECTED")

        # TODO osidb_session.affects does not support include_fields, do it manually for now
        affect_critical_res = requests.get(
            f"{OSIDB_API_URL}/osidb/api/v1/affects?affectedness=AFFECTED&impact=CRITICAL&include_fields=ps_component,ps_module,impact&limit=10000"  # noqa
        )
        affects_critical = affect_critical_res.json()
        critical_components = {}
        critical_products = {}
        for affect in affects_critical["results"]:
            component_name = affect.get("ps_component", "no_name")
            if component_name not in critical_components:
                critical_components[component_name] = 0
            critical_components[component_name] += 1

            product_version_name = affect.get("ps_module", "no_name")
            if product_version_name not in critical_products:
                critical_products[product_version_name] = 0
            critical_products[product_version_name] += 1

        top_critical_component = max(critical_components, key=critical_components.get)  # type: ignore # noqa
        top_critical_product = max(critical_products, key=critical_products.get)  # type: ignore

        affect_important_res = requests.get(
            f"{OSIDB_API_URL}/osidb/api/v1/affects?affectedness=AFFECTED&impact=IMPORTANT&include_fields=ps_component,ps_module,impact&limit=10000"  # noqa
        )
        affects_important = affect_important_res.json()
        important_components = {}
        important_products = {}
        for affect in affects_important["results"]:
            component_name = affect.get("ps_component", "no_name")
            if component_name not in important_components:
                important_components[component_name] = 0
            important_components[component_name] += 1

            product_version_name = affect.get("ps_module", "no_name")
            if product_version_name not in important_products:
                important_products[product_version_name] = 0
            important_products[product_version_name] += 1
        top_important_component = max(important_components, key=important_components.get)  # type: ignore # noqa
        top_important_product = max(important_products, key=important_products.get)  # type: ignore # noqa

        affect_moderate_res = requests.get(
            f"{OSIDB_API_URL}/osidb/api/v1/affects?affectedness=AFFECTED&impact=MODERATE&include_fields=ps_component,ps_module,impact&limit=10000"  # noqa
        )
        affects_moderate = affect_moderate_res.json()
        moderate_components = {}
        moderate_products = {}
        for affect in affects_moderate["results"]:
            component_name = affect.get("ps_component", "no_name")
            if component_name not in moderate_components:
                moderate_components[component_name] = 0
            moderate_components[component_name] += 1

            product_version_name = affect.get("ps_module", "no_name")
            if product_version_name not in moderate_products:
                moderate_products[product_version_name] = 0
            moderate_products[product_version_name] += 1
        top_moderate_component = max(moderate_components, key=moderate_components.get)  # type: ignore # noqa
        top_moderate_product = max(moderate_products, key=moderate_products.get)  # type: ignore

        affect_low_res = requests.get(
            f"{OSIDB_API_URL}/osidb/api/v1/affects?affectedness=AFFECTED&impact=LOW&include_fields=ps_component,ps_module,impact&limit=10000"  # noqa
        )
        affects_low = affect_low_res.json()
        low_components = {}
        low_products = {}
        for affect in affects_low["results"]:
            component_name = affect.get("ps_component", "no_name")
            if component_name not in low_components:
                low_components[component_name] = 0
            low_components[component_name] += 1

            product_version_name = affect.get("ps_module", "no_name")
            if product_version_name not in low_products:
                low_products[product_version_name] = 0
            low_products[product_version_name] += 1
        top_low_component = max(low_components, key=low_components.get)  # type: ignore
        top_low_product = max(low_products, key=low_products.get)  # type: ignore

        critical = {
            "affected": affects_critical["count"],
            "top_component": top_critical_component,
            "top_product": top_critical_product,
        }
        important = {
            "affected": affects_important["count"],
            "top_component": top_important_component,
            "top_product": top_important_product,
        }
        moderate = {
            "affected": affects_moderate["count"],
            "top_component": top_moderate_component,
            "top_product": top_moderate_product,
        }
        low = {
            "affected": affects_low["count"],
            "top_component": top_low_component,
            "top_product": top_low_product,
        }

        if show_components:
            critical["components"] = critical_components
            important["components"] = important_components
            moderate["components"] = moderate_components
            low["components"] = low_components

        if show_products:
            critical["products"] = critical_products
            important["products"] = important_products
            moderate["products"] = moderate_products
            low["products"] = low_products

        report = {
            "title": "Example Affects report",
            "ts": str(datetime.now()),
            "total_affects": affects.count,
            "total_affected": affects_affected.count,
            "critical": critical,
            "important": important,
            "moderate": moderate,
            "low": low,
        }

        if purl:
            report["purl"] = purl
        if component_name:
            report["component_name"] = component_name
        if ofuri:
            report["product_ofuri"] = ofuri
        if product_name:
            report["product"] = product_name

        return {
            "title": "Example Affects report",
            "ts": str(datetime.now()),
            "total_affects": affects.count,
            "total_affected": affects_affected.count,
            "critical": critical,
            "important": important,
            "moderate": moderate,
            "low": low,
        }



"""
exploits plugin:


"""
import logging

import click

logger = logging.getLogger("griffon")

api_url = ""


# define autocomplete for service names
@click.group(help="(UNDER DEV) exploits plugin")
@click.pass_context
def plugins(ctx):
    """exploits plugin"""
    pass


@plugins.command()
@click.option("--name", "service_name")
@click.pass_context
def stub(ctx, service_name):
    """ """
    pass



import pytest
from click.testing import CliRunner

from griffon.cli import cli

pytestmark = pytest.mark.unit


def test_osidb():
    runner = CliRunner()
    result = runner.invoke(cli, ["manage", "osidb"])
    assert result.exit_code == 0


def test_corgi():
    runner = CliRunner()
    result = runner.invoke(cli, ["manage", "corgi"])
    assert result.exit_code == 0



"""griffon Python Package

This setup.py file uses setuptools to install the `griffon` package
"""

from setuptools import setup

with open("README.md") as f:
    readme = f.read()

with open("griffon/__init__.py") as f:
    for line in f:
        if line.startswith("__version__"):
            delim = '"' if '"' in line else "'"
            version = line.split(delim)[1]
            break
    else:
        raise RuntimeError("Unable to find version string.")

setup(
    name="griffon",
    version=version,
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/RedHatProductSecurity/griffon",
    description="Red Hat Product Security CLI",
    packages=[
        "griffon",
        "griffon/commands",
        "griffon/commands/plugins",
        "griffon/services",
        "griffon/autocomplete",
    ],
    install_requires=[
        "click",
        "click-completion",
        "rich",
        "osidb-bindings",
        "component-registry-bindings",
        "packageurl-python",
    ],
    entry_points={"console_scripts": ["griffon=griffon.cli:cli"]},
    author="James Fuller, Red Hat Product Security",
    license="MIT",
    classifiers=[
        "Topic :: Security",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
    ],
)



"""
vuln.go.dev plugin:


"""
import click
import requests

from griffon.output import cprint

api_url = "https://vuln.go.dev"


@click.group()
@click.pass_context
def plugins(ctx):
    """vuln.go.dev plugin"""
    pass


@plugins.command()
@click.option("--id", "go_id", help="go vuln ID (ex. GO-2022-0189")
@click.option("--cve-id", "cve_id", help="retrieve by CVE_ID alias")
@click.pass_context
def get(ctx, go_id, cve_id):
    """Search go.vuln.dev by go vuln ID"""
    if not go_id and not cve_id:
        click.echo(ctx.get_help())
        exit(0)

    if cve_id:
        res = requests.get(f"https://pkg.go.dev/search?q={cve_id}")
        go_id = res.url.split("/")[-1]
    if go_id:
        res = requests.get(f"{api_url}/ID/{go_id}.json")
        cprint(res.json(), ctx=ctx)


@plugins.command()
@click.option(
    "-q",
    "search_term",
)
@click.pass_context
def search(ctx, search_term):
    """Search go.vuln.dev"""
    if not search_term:
        click.echo(ctx.get_help())
        exit(0)
    click.launch(f"https://pkg.go.dev/search?q={search_term}")



"""

"""
import logging

import click

from griffon import CorgiService, OSIDBService, progress_bar
from griffon.autocomplete import get_cve_ids, get_product_version_names
from griffon.commands.entities import (
    get_component_manifest,
    get_product_stream_manifest,
    get_product_stream_names,
    get_product_stream_ofuris,
    list_components,
)
from griffon.commands.process import generate_affects_for_component_process
from griffon.commands.reports import generate_affects_report
from griffon.output import cprint
from griffon.services import QueryService, core_queries  # , exp

logger = logging.getLogger("griffon")

query_service = QueryService()


@click.group(name="service", help="Service operations.")
@click.pass_context
def queries_grp(ctx):
    """Query operations."""
    pass


queries_grp.add_command(generate_affects_report)
queries_grp.add_command(generate_affects_for_component_process)


@queries_grp.command(
    name="product-summary",
    help="Get Product summaries.",
)
@click.argument(
    "product_stream_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_stream_names,
)
@click.option(
    "--ofuri",
    "ofuri",
    type=click.STRING,
    shell_complete=get_product_stream_ofuris,
    help="UNDER DEVELOPMENT",
)
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@click.pass_context
@progress_bar
def get_product_summary(ctx, product_stream_name, ofuri, strict_name_search):
    """get product stream."""
    if not product_stream_name and not ofuri:
        click.echo(ctx.get_help())
        exit(0)
    q = query_service.invoke(core_queries.product_stream_summary, ctx.params)
    cprint(q, ctx=ctx)


@queries_grp.command(
    name="products-contain-component",
    help="List Products containing (latest) Component.",
)
@click.argument("component_name", required=False)
@click.option(
    "--purl", help="Component purl, needs to be in quotes (ex. 'pkg:rpm/python-pyjwt@1.7.1')"
)
@click.option(
    "--arch",
    default="src",
    type=click.Choice(CorgiService.get_component_arches()),
)
@click.option(
    "--namespace", default=None, type=click.Choice(CorgiService.get_component_namespaces())
)
@click.option("--type", "component_type", type=click.Choice(CorgiService.get_component_types()))
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@click.option(
    "--affect",
    "affect_mode",
    is_flag=True,
    default=False,
    help="(Under development) Generate Affects.",
)
@click.option(
    "--search-deps",
    "search_deps",
    is_flag=True,
    default=False,
    help="(Under development) Search Component dependencies.",
)
@click.pass_context
@progress_bar
def get_product_contain_component(
    ctx,
    component_name,
    purl,
    arch,
    namespace,
    component_type,
    strict_name_search,
    affect_mode,
    search_deps,
):
    """List products of a latest component."""
    if not purl and not component_name:
        click.echo(ctx.get_help())
        click.echo("")
        click.echo("Must supply --name or --purl.")
        exit(0)
    if component_name:
        q = query_service.invoke(core_queries.products_containing_component_query, ctx.params)
        cprint(q, ctx=ctx)
    if purl:
        q = query_service.invoke(
            core_queries.products_containing_specific_component_query, ctx.params
        )
        cprint(q, ctx=ctx)


@queries_grp.command(
    name="components-contain-component",
    help="List Components contain component.",
)
@click.argument("component_name", required=False)
@click.option("--purl")
@click.option("--type", "component_type", type=click.Choice(CorgiService.get_component_types()))
@click.option("--version", "component_version")
@click.option(
    "--arch",
    "component_arch",
    type=click.Choice(CorgiService.get_component_arches()),
    help="Default arch=src.",
)
@click.option("--namespace", type=click.Choice(CorgiService.get_component_namespaces()))
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@click.pass_context
@progress_bar
def get_component_contain_component(
    ctx,
    component_name,
    purl,
    component_type,
    component_version,
    component_arch,
    namespace,
    strict_name_search,
):
    """List components that contain component."""
    if not component_name and not purl:
        click.echo(ctx.get_help())
        exit(0)
    if component_name:
        q = query_service.invoke(core_queries.components_containing_component_query, ctx.params)
        cprint(
            q,
            ctx=ctx,
        )
    if purl:
        q = query_service.invoke(
            core_queries.components_containing_specific_component_query, ctx.params
        )
        cprint(
            q,
            ctx=ctx,
        )


@queries_grp.command(
    name="product-manifest",
    help="Get Product manifest (includes Root Components and all dependencies).",
)
@click.argument("product_stream_name", required=False, shell_complete=get_product_stream_names)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_stream_ofuris)
@click.option(
    "--spdx-json",
    "spdx_json_format",
    is_flag=True,
    default=False,
    help="Generate spdx manifest (json).",
)
@click.pass_context
def get_product_manifest_query(ctx, product_stream_name, ofuri, spdx_json_format):
    """List components of a specific product version."""
    if not ofuri and not product_stream_name:
        click.echo(ctx.get_help())
        exit(0)

    if spdx_json_format:
        ctx.ensure_object(dict)
        ctx.obj["FORMAT"] = "json"

    cond = {}
    if ofuri:
        cond["ofuri"] = ofuri
    if product_stream_name:
        cond["product_stream_name"] = product_stream_name
    ctx.invoke(get_product_stream_manifest, **cond)


@queries_grp.command(
    name="product-components",
    help="List LATEST Root Components of Product.",
)
@click.pass_context
@click.argument("product_stream_name", required=False, shell_complete=get_product_stream_names)
@click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_stream_ofuris)
def get_product_latest_components_query(ctx, product_stream_name, ofuri):
    """List components of a specific product version."""
    if not ofuri and not product_stream_name:
        click.echo(ctx.get_help())
        exit(0)
    cond = {}
    if ofuri:
        cond["ofuri"] = ofuri
    if product_stream_name:
        # lookup ofuri
        q = query_service.invoke(core_queries.product_stream_summary, ctx.params)
        ofuri = q[0]["ofuri"]
        cond["ofuri"] = ofuri
    ctx.invoke(list_components, **cond)


# TODO- can always use entity commands to achieve this, deactivating for the time being
# @queries_grp.command(
#     name="product-all-components",
#     help="List ALL Components of Product.",
# )
# @click.argument("product_stream_name", required=False, shell_complete=get_product_stream_names)
# @click.pass_context
# @click.option("--ofuri", "ofuri", type=click.STRING, shell_complete=get_product_stream_ofuris)
# def get_product_all_components_query(ctx, product_stream_name, ofuri):
#     """List components of a specific product stream."""
#     if not ofuri and not product_stream_name:
#         click.echo(ctx.get_help())
#         exit(0)
#     cond = {}
#     if ofuri:
#         cond["product_stream_ofuri"] = ofuri
#     if product_stream_name:
#         # lookup ofuri
#         q = query_service.invoke(core_queries.product_stream_summary, ctx.params)
#         ofuri = q[0]["ofuri"]
#         cond["product_stream_ofuri"] = ofuri
#     ctx.invoke(list_components, **cond)


@queries_grp.command(
    name="component-manifest",
    help="Get Component manifest.",
)
@click.option("--uuid", "component_uuid")
@click.option("--purl", help="Component Purl (must be quoted).")
@click.option(
    "--spdx-json",
    "spdx_json_format",
    is_flag=True,
    default=False,
    help="Generate spdx manifest (json).",
)
@click.pass_context
def retrieve_component_manifest(ctx, component_uuid, purl, spdx_json_format):
    """Retrieve component manifest."""
    if not component_uuid and not purl:
        click.echo(ctx.get_help())
        exit(0)
    if spdx_json_format:
        ctx.ensure_object(dict)
        ctx.obj["FORMAT"] = "json"
    cond = {}
    if component_uuid:
        cond["component_uuid"] = component_uuid
    if purl:
        cond["purl"] = purl
    ctx.invoke(get_component_manifest, **cond)


@queries_grp.command(
    name="components-affected-by-flaw",
    help="List Components affected by Flaw.",
)
@click.argument("cve_id", required=False, type=click.STRING, shell_complete=get_cve_ids)
@click.option(
    "--affectedness",
    help="Filter by Affect affectedness.",
    type=click.Choice(OSIDBService.get_affect_affectedness()),
)
@click.option(
    "--resolution",
    "affect_resolution",
    help="Filter by Affect resolution.",
    type=click.Choice(OSIDBService.get_affect_resolution()),
)
@click.option(
    "--impact",
    "affect_impact",
    help="Filter by Affect impact.",
    type=click.Choice(OSIDBService.get_affect_impact()),
)
@click.option(
    "--type",
    "component_type",
    type=click.Choice(CorgiService.get_component_types()),
    help="Filter by Component type.",
)
@click.option(
    "--namespace",
    type=click.Choice(CorgiService.get_component_namespaces()),
    help="filter by Component namespace.",
)
@click.pass_context
@progress_bar
def components_affected_by_specific_cve_query(
    ctx, cve_id, affectedness, affect_resolution, affect_impact, component_type, namespace
):
    """List components affected by specific CVE."""
    if not cve_id:
        click.echo(ctx.get_help())
        exit(0)
    q = query_service.invoke(core_queries.components_affected_by_specific_cve_query, ctx.params)
    cprint(
        q,
        ctx=ctx,
    )


@queries_grp.command(
    name="products-affected-by-flaw",
    help="List Products affected by Flaw.",
)
@click.option("--cve-id", shell_complete=get_cve_ids)
@click.pass_context
@progress_bar
def product_versions_affected_by_cve_query(ctx, cve_id):
    """List Products affected by a CVE."""
    if not cve_id:
        click.echo(ctx.get_help())
        exit(0)
    q = query_service.invoke(
        core_queries.products_versions_affected_by_specific_cve_query, ctx.params
    )
    cprint(q, ctx=ctx)


@queries_grp.command(name="component-flaws", help="List Flaws affecting a Component.")
@click.argument("component_name", required=False)
@click.option("--purl")
@click.option(
    "--flaw-state",
    "flaw_state",
    help="Filter by Flaw state.",
    type=click.Choice(OSIDBService.get_flaw_states()),
)
@click.option(
    "--flaw-impact",
    "flaw_impact",
    help="Filter by Flaw impact.",
    type=click.Choice(OSIDBService.get_flaw_impacts()),
)
@click.option(
    "--flaw-resolution",
    "flaw_resolution",
    help="Filter by Flaw resolution.",
    type=click.Choice(OSIDBService.get_flaw_resolutions()),
)
@click.option(
    "--affectedness",
    help="Filter by Affect affectedness.",
    type=click.Choice(OSIDBService.get_affect_affectedness()),
)
@click.option(
    "--affect-resolution",
    "affect_resolution",
    help="Filter by Affect resolution.",
    type=click.Choice(OSIDBService.get_affect_resolution()),
)
@click.option(
    "--affect-impact",
    "affect_impact",
    help="Filter by Affect impact.",
    type=click.Choice(OSIDBService.get_affect_impact()),
)
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@click.pass_context
@progress_bar
def cves_for_specific_component_query(
    ctx,
    component_name,
    purl,
    flaw_state,
    flaw_impact,
    flaw_resolution,
    affectedness,
    affect_resolution,
    affect_impact,
    strict_name_search,
):
    """List flaws of a specific component."""
    if not purl and not component_name:
        click.echo(ctx.get_help())
        exit(0)

    q = query_service.invoke(core_queries.cves_for_specific_component_query, ctx.params)
    cprint(q, ctx=ctx)


@queries_grp.command(
    name="product-flaws",
    help="List Flaws affecting a Product.",
)
@click.argument(
    "product_version_name",
    required=False,
    type=click.STRING,
    shell_complete=get_product_version_names,
)
@click.option("--ofuri")
@click.option(
    "--flaw-state",
    "flaw_state",
    help="Filter by Flaw state.",
    type=click.Choice(OSIDBService.get_flaw_states()),
)
@click.option(
    "--flaw-impact",
    "flaw_impact",
    help="Filter by Flaw impact.",
    type=click.Choice(OSIDBService.get_flaw_impacts()),
)
@click.option(
    "--flaw-resolution",
    "flaw_resolution",
    help="Filter by Flaw resolution.",
    type=click.Choice(OSIDBService.get_flaw_resolutions()),
)
@click.option(
    "--affectedness",
    help="Filter by Affect affectedness.",
    type=click.Choice(OSIDBService.get_affect_affectedness()),
)
@click.option(
    "--affect-resolution",
    "affect_resolution",
    help="Filter by Affect resolution.",
    type=click.Choice(OSIDBService.get_affect_resolution()),
)
@click.option(
    "--affect-impact",
    "affect_impact",
    help="Filter by Affect impact.",
    type=click.Choice(OSIDBService.get_affect_impact()),
)
@click.option(
    "-s",
    "strict_name_search",
    is_flag=True,
    default=False,
    help="Strict search, exact match of component name.",
)
@click.pass_context
def cves_for_specific_product_query(
    ctx,
    product_version_name,
    ofuri,
    flaw_state,
    flaw_impact,
    flaw_resolution,
    affectedness,
    affect_impact,
    affect_resolution,
    strict_name_search,
):
    """List flaws of a specific product."""
    if not product_version_name and not ofuri:
        click.echo(ctx.get_help())
        exit(0)
    q = query_service.invoke(core_queries.cves_for_specific_product_query, ctx.params)
    cprint(q, ctx=ctx)



import logging

import requests

from griffon import CORGI_API_URL, OSIDB_API_URL

logger = logging.getLogger("griffon")


def get_product_version_ofuris(ctx, param, incomplete):
    response = requests.get(
        f"{CORGI_API_URL}/api/v1/product_versions?limit=1000&include_fields=ofuri&re_ofuri={incomplete}"  # noqa
    )
    return [k["ofuri"] for k in response.json()["results"] if incomplete in k["ofuri"]]


def get_product_version_names(ctx, param, incomplete):
    response = requests.get(
        f"{CORGI_API_URL}/api/v1/product_versions?limit=1000&include_fields=name&re_name={incomplete}"  # noqa
    )
    return [k["name"] for k in response.json()["results"] if incomplete in k["name"]]


def get_product_stream_ofuris(ctx, param, incomplete):
    payload = {"limit": 2000, "include_fields": "ofuri", "re_ofuri": incomplete}
    # if ctx.obj["SHOW_INACTIVE"]:
    #     payload["active"] = "all"
    response = requests.get(
        f"{CORGI_API_URL}/api/v1/product_streams",
        params=payload,
    )
    return [k["ofuri"] for k in response.json()["results"] if incomplete in k["ofuri"]]


def get_product_stream_names(ctx, param, incomplete):
    payload = {"limit": 2000, "include_fields": "name", "re_name": incomplete}
    # if ctx.obj["SHOW_INACTIVE"]:
    #     payload["active"] = "all"
    response = requests.get(
        f"{CORGI_API_URL}/api/v1/product_streams",
        params=payload,
    )
    names = response.json()["results"]
    return [k["name"] for k in names if "name" in k]


def get_component_names(ctx, param, incomplete):
    response = requests.get(
        f"{CORGI_API_URL}/api/v1/components?include_fields=name&re_name={incomplete}&limit=100"  # noqa
    )
    names = response.json()["results"]
    return list(set([k["name"] for k in names if "name" in k]))


def get_component_purls(ctx, param, incomplete):
    response = requests.get(
        f"{CORGI_API_URL}/api/v1/components?include_fields=purl&re_purl={incomplete}&limit=20"  # noqa
    )
    return [k["purl"] for k in response.json()["results"]]


def get_cve_ids(ctx, param, incomplete):
    """TODO - the following is not ideal for autocomplete lookup - need to investigate"""
    response = requests.get(
        f"{OSIDB_API_URL}/osidb/api/v1/flaws?limit=10&re_cve_id={incomplete}&include_fields=cve_id"  # noqa
    )
    return [k["cve_id"] for k in response.json()["results"] if incomplete in k["cve_id"]]
